Program HanoiSim;
{ Simulates generated solutios to the Towers of Hanoi problem.
  Input taken from file solution.txt generated by HANOIALL.PAS
  or created otherwise. Ignores 2nd line with number of solutions.
  First error reported, then program stops. }
Uses Crt, Dataunit, Listunit;

Const fname = 'solution.txt';

Var Theight,Counter: integer;
    c: char;
    s: string;
    lasize: word;
    l: Listpoint;
    na,np1,np2: Nodepoint;
    f: text;

Procedure Simulate(g:string; height: integer);
{ Simulates one solution to the Towers of Hanoi.
  g is the solution string - sequence of moves
  separated by any character (space). No leading
  space. Height is the tower height.
  Halts after first reported error. }

Var L,C,R : array[1..10] of integer;
    nl,nc,nr,i,p,size,d : integer;
    from,too,dc : char;
    move : string;

 Procedure error(n:Integer); { Reporting various errors }
 Var s : string;
 Begin
   case n of
     1: s := 'empty platform L to move from';
     2: s := 'disk mismatch on platform L to move from';
     3: s := 'move L - C : disk ' + dc + ' to smaller disk ' + chr(48+C[nc]);
     4: s := 'move L - R : disk ' + dc + ' to smaller disk ' + chr(48+R[nr]);

     5: s := 'empty platform C to move from';
     6: s := 'disk mismatch on platform C to move from';
     7: s := 'move C - L : disk ' + dc + ' to smaller disk ' + chr(48+L[nl]);
     8: s := 'move C - R : disk ' + dc + ' to smaller disk ' + chr(48+R[nr]);

     9: s := 'empty platform C to move from';
    10: s := 'disk mismatch on platform C to move from';
    11: s := 'move C - L : disk ' + dc + ' to smaller disk ' + chr(48+L[nl]);
    12: s := 'move C - R : disk ' + dc + ' to smaller disk ' + chr(48+R[nr]);

    15: s := 'wrong platform to move from';
    16: s := 'wrong platform to move to from L';
    17: s := 'wrong platform to move to from C';
    18: s := 'wrong platform to move to from R';

    20: s := 'wrong final status';
   end;
   TextColor(LightRed);
   Writeln('Solution ',Counter,' Move ',move,' Error: ',s);
   Readln;
   Halt;
 End;

Begin
   for i:=1 to height do L[i] := height-i+1;  { Game initialization }
   nl := height;                              { Biggest disk at index 1 }
   nc := 0;
   nr := 0;
   size := length(g);   { String size }
   p := 1;              { Current pointer in g - 1st character of the move }
   while p<size do begin
      move := copy(g,p,3);           { Move triple }
      dc := g[p];                    { Moving disk dc }
      d := ord(g[p]) - 48;           { Moving disk d (number) }
      from := upcase(g[p+1]);        { Platform from }
      too := upcase(g[p+2]);         { Platform to }
      case from of
       'L': begin                        { Move from L }
             if nl=0 then error(1);
             if L[nl]<>d then error(2);
             dec(nl);
             case too of
              'C': begin                 { Move L - C }
                    if nc=0 then begin   { Target empty - OK }
                      nc := 1;
                      C[1] := d;
                    end else begin
                      if C[nc]<d then error(3);
                      inc(nc);
                      C[nc] := d;
                    end
                   end;
              'R': begin                 { Move L - R }
                    if nr=0 then begin   { Target empty - OK }
                      nr := 1;
                      R[1] := d;
                    end else begin
                      if R[nr]<d then error(4);
                      inc(nr);
                      R[nr] := d;
                    end
                   end;
              else
                error(16);
             end{case};
            end{L};
       'C': begin                        { Move from C }
             if nc=0 then error(5);
             if C[nc]<>d then error(6);
             dec(nc);
             case too of
              'L': begin                 { Move C - L }
                    if nl=0 then begin   { Target empty - OK }
                      nl := 1;
                      L[1] := d;
                    end else begin
                      if L[nl]<d then error(7);
                      inc(nl);
                      L[nl] := d;
                    end
                   end;
              'R': begin                 { Move C - R }
                    if nr=0 then begin   { Target empty - OK }
                      nr := 1;
                      R[1] := d;
                    end else begin
                      if R[nr]<d then error(8);
                      inc(nr);
                      R[nr] := d;
                    end
                   end;
              else
                error(17);
             end{case};
            end{C};
       'R': begin                        { Move from R }
             if nr=0 then error(9);
             if R[nr]<>d then error(10);
             dec(nr);
             case too of
              'L': begin                 { Move R - L }
                    if nl=0 then begin   { Target empty - OK }
                      nl := 1;
                      L[1] := d;
                    end else begin
                      if L[nl]<d then error(11);
                      inc(nl);
                      L[nl] := d;
                    end
                   end;
              'C': begin                 { Move R - C }
                    if nc=0 then begin   { Target empty - OK }
                      nc := 1;
                      C[1] := d;
                    end else begin
                      if C[nc]<d then error(12);
                      inc(nc);
                      C[nc] := d;
                    end
                   end;
              else
                error(18);
             end{case};
            end{R};
       else
          error(15);
      end{case from};
      p := p+4;
   end{while};
   if nl<>0 then error(20);
   if nc<>0 then error(20);  { If these are 0, the tower must be on R }
End;

Begin
   Assign(f,fname);
   Reset(f);
   ClrScr;
   TextColor(LightBlue);
   Writeln('Creating the list of solutions');
   l := Newlist;
   Readln(f,Theight);
   Readln(f,Counter);
   while not eof(f) do begin
     Read(f,Counter);
     Read(f,c);
     new(na);
     Readln(f,na^.el.s);
     Inslast(na,l);
   end;
   TextColor(Green);
   lasize := Getsize(l);
   Writeln('List created, number of solutions: ',lasize);
   Counter := 1;
   while not Empty(l) do begin
     na := Remfirst(l);
     Simulate(na^.el.s, Theight);
     Writeln('Solution ',Counter,' : simulation OK');
     Inc(Counter);
   end{while};
   Close(f);
   TextColor(Yellow);
   Writeln('Simulation finished.');
   Readln;
End.
